<?php /** @noinspection PhpUndefinedClassInspection */
/**
 * Couponer for Elementor
 * Discount Coupons Widget for Elementor Builder
 * Exclusively on https://1.envato.market/couponer-elementor
 *
 * @encoding        UTF-8
 * @version         1.1.5
 * @copyright       (C) 2018 - 2022 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Cherviakov Vlad (vladchervjakov@gmail.com)
 * @support         help@merkulov.design
 **/

namespace Merkulove\CouponerElementor;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

use Exception;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Text_Shadow;
use Elementor\Group_Control_Typography;
use Elementor\Group_Control_Css_Filter;
use Elementor\Core\Schemes\Typography;
use Elementor\Utils;
use Elementor\Group_Control_Box_Shadow;
use Elementor\Repeater;
use Elementor\Group_Control_Border;
use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Merkulove\CouponerElementor\Unity\Plugin as UnityPlugin;

/** @noinspection PhpUnused */
/**
 * Couponer - Custom Elementor Widget.
 **/
class couponer_elementor extends Widget_Base {

    /**
     * Use this to sort widgets.
     * A smaller value means earlier initialization of the widget.
     * Can take negative values.
     * Default widgets and widgets from 3rd party developers have 0 $mdp_order
     **/
    public $mdp_order = 1;

    /**
     * Widget base constructor.
     * Initializing the widget base class.
     *
     * @access public
     * @throws Exception If arguments are missing when initializing a full widget instance.
     * @param array      $data Widget data. Default is an empty array.
     * @param array|null $args Optional. Widget default arguments. Default is null.
     *
     * @return void
     **/
    public function __construct( $data = [], $args = null ) {

        parent::__construct( $data, $args );

        wp_register_style(
            'mdp-couponer-elementor-admin',
            UnityPlugin::get_url() . 'src/Merkulove/Unity/assets/css/elementor-admin' . UnityPlugin::get_suffix() . '.css',
            [],
            UnityPlugin::get_version()
        );

        wp_register_style(
            'mdp-couponer-elementor',
            UnityPlugin::get_url() . 'css/couponer-elementor' . UnityPlugin::get_suffix() . '.css',
            [],
            UnityPlugin::get_version()
        );

	    wp_register_script(
            'mdp-couponer-elementor',
            UnityPlugin::get_url() . 'js/couponer-elementor' . UnityPlugin::get_suffix() . '.js',
            [ 'jquery', 'elementor-frontend' ],
            UnityPlugin::get_version(),
            true
        );


    }

    /**
     * Return a widget name.
     *
     * @return string
     **/
    public function get_name() {
        return 'mdp-couponer-elementor';
    }

    /**
     * Return the widget title that will be displayed as the widget label.
     *
     * @return string
     **/
    public function get_title() {
        return esc_html__( 'Couponer', 'couponer-elementor' );
    }

    /**
     * Set the widget icon.
     *
     * @return string
     */
    public function get_icon() {

        return 'mdp-couponer-elementor-widget-icon';

    }

    /**
     * Set the category of the widget.
     *
     * @return array with category names
     **/
    public function get_categories() {

        return [ 'general' ];

    }

    /**
     * Get widget keywords. Retrieve the list of keywords the widget belongs to.
     *
     * @access public
     *
     * @return array Widget keywords.
     **/
    public function get_keywords() {

        return [ 'Merkulove', 'Couponer', 'coupon', 'code', 'sale', 'discount', 'woocommerce' ];

    }

    /**
     * Get style dependencies.
     * Retrieve the list of style dependencies the widget requires.
     *
     * @access public
     *
     * @return array Widget styles dependencies.
     **/
    public function get_style_depends() {

        return [ 'mdp-couponer-elementor', 'mdp-couponer-elementor-admin' ];

    }

	/**
	 * Get script dependencies.
	 * Retrieve the list of script dependencies the element requires.
	 *
	 * @access public
     *
	 * @return array Element scripts dependencies.
	 **/
	public function get_script_depends() {

		return [ 'mdp-couponer-elementor' ];

    }

    /**
     * Render group of controls for styling elements
     *
     * @param $name_section
     * @param string $css_class
     * @param array $condition
     * @param array $display
     */
    public function get_settings_group( $name_section, $css_class, $condition = [], $display = [] ) {

        /** Width */
        if ( isset( $display[ 'width' ] ) && $display[ 'width' ] === true ) {
            $this->add_responsive_control(
                'couponer_' . $name_section . '_width',
                [
                    'label'      => esc_html__( 'Width', 'couponer-elementor' ),
                    'type'       => Controls_Manager::SLIDER,
                    'size_units' => [ 'px', '%' ],
                    'default'    => [
                        'unit' => '%',
                        'size' => 100,
                    ],
                    'range'      => [
                        '%' => [
                            'min'  => 0,
                            'max'  => 100,
                            'step' => 1,
                        ],
                    ],
                    'selectors'  => [
                        '{{WRAPPER}} ' . $css_class => 'width: {{size}}{{unit}};',
                    ],
                    'condition'  => $condition,
                    'separator'  => 'after'
                ]
            );
        }

        /** Margin. */
        if ( isset($display['margin']) && $display['margin'] === true ) {
            $this->add_responsive_control(
                'couponer_' . $name_section . '_margin',
                [
                    'label'      => esc_html__( 'Margin', 'couponer-elementor' ),
                    'type'       => Controls_Manager::DIMENSIONS,
                    'size_units' => [ 'px', '%', 'em' ],
                    'selectors'  => [
                        '{{WRAPPER}} ' . $css_class => 'margin: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
                    ],
                    'toggle'     => true,
                    'condition'  => $condition,
                ]
            );
        }

        /** Padding. */
        if ( isset($display['padding']) && $display['padding'] === true ) {
            $this->add_responsive_control(
                'couponer_' . $name_section . '_padding',
                [
                    'label'      => esc_html__( 'Padding', 'couponer-elementor' ),
                    'type'       => Controls_Manager::DIMENSIONS,
                    'size_units' => [ 'px', '%', 'em' ],
                    'selectors'  => [
                        '{{WRAPPER}} ' . $css_class => 'padding: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
                    ],
                    'toggle'     => true,
                    'separator'  => 'after',
                    'condition'  => $condition,
                ]
            );
        }

        /** Text Color. */
        if ( isset($display['color']) && $display['color'] === true ) {
            $this->add_control(
                'couponer_' . $name_section . '_text_color',
                [
                    'label'     => esc_html__( 'Color', 'couponer-elementor' ),
                    'type'      => Controls_Manager::COLOR,
                    'selectors' => [
                        '{{WRAPPER}} ' . $css_class => 'color: {{VALUE}}',
                    ],
                    'condition' => $condition,
                ]
            );
        }

        /** Typography */
        if ( isset($display['typography']) && $display['typography'] === true ) {
            $this->add_group_control(
                Group_Control_Typography::get_type(),
                [
                    'name'      => 'couponer_' . $name_section . '_typography',
                    'label'     => esc_html__( 'Typography', 'couponer-elementor' ),
                    'scheme'    => Typography::TYPOGRAPHY_1,
                    'selector'  => '{{WRAPPER}} ' . $css_class,
                    'condition' => $condition,
                ]
            );
        }

        /** Text Shadow. */
        if ( isset($display['text-shadow']) && $display['text-shadow'] === true ) {
            $this->add_group_control(
                Group_Control_Text_Shadow::get_type(),
                [
                    'name'      => 'couponer_' . $name_section . '_text_shadow',
                    'label'     => esc_html__( 'Text Shadow', 'couponer-elementor' ),
                    'selector'  => '{{WRAPPER}} ' . $css_class,
                    'condition' => $condition,
                ]
            );
        }

        /** Alignment. */
        if ( isset($display['alignment']) && $display['alignment'] === true ) {
            $this->add_responsive_control(
                'couponer_' . $name_section . '_align',
                [
                    'label'     => esc_html__( 'Alignment', 'couponer-elementor' ),
                    'type'      => Controls_Manager::CHOOSE,
                    'options'   => [
                        'left'    => [
                            'title' => esc_html__( 'Left', 'couponer-elementor' ),
                            'icon'  => 'fa fa-align-left',
                        ],
                        'center'  => [
                            'title' => esc_html__( 'Center', 'couponer-elementor' ),
                            'icon'  => 'fa fa-align-center',
                        ],
                        'right'   => [
                            'title' => esc_html__( 'Right', 'couponer-elementor' ),
                            'icon'  => 'fa fa-align-right',
                        ],
                        'justify' => [
                            'title' => esc_html__( 'Justify', 'couponer-elementor' ),
                            'icon'  => 'fa fa-align-justify',
                        ],
                    ],
                    'default'   => 'center',
                    'selectors' => [
                        '{{WRAPPER}} ' . $css_class => 'text-align: {{header_align}};',
                    ],
                    'toggle'    => true,
                    'condition' => $condition,
                ]
            );
        }

        /** Background. */
        if ( isset($display['background']) && $display['background'] === true ) {
            $this->add_group_control(
                Group_Control_Background::get_type(),
                [
                    'name'      => 'couponer_' . $name_section . '_background',
                    'label'     => esc_html__( 'Background', 'couponer-elementor' ),
                    'types'     => [ 'classic', 'gradient' ],
                    'separator' => 'before',
                    'selector'  => '{{WRAPPER}} ' . $css_class,
                    'condition' => $condition,
                ]
            );
        }

        /** Border. */
        if ( isset($display['border']) && $display['border'] === true ) {
            $this->add_group_control(
                Group_Control_Border::get_type(),
                [
                    'name'      => 'couponer_' . $name_section . '_border',
                    'label'     => esc_html__( 'Border', 'couponer-elementor' ),
                    'selector'  => '{{WRAPPER}} ' . $css_class,
                    'separator' => 'before',
                    'condition' => $condition,
                ]
            );
        }

        /** Border Radius. */
        if ( isset($display['border-radius']) && $display['border-radius'] === true ) {
            $this->add_responsive_control(
                'couponer_' . $name_section . '_border_radius',
                [
                    'label'      => esc_html__( 'Border Radius', 'couponer-elementor' ),
                    'type'       => Controls_Manager::DIMENSIONS,
                    'size_units' => [ 'px', '%', 'em' ],
                    'selectors'  => [
                        '{{WRAPPER}} ' . $css_class => 'border-radius: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
                    ],
                    'separator'  => 'after',
                    'toggle'     => true,
                    'condition'  => $condition,
                ]
            );
        }

        /** Box Shadow. */
        if ( isset($display['box-shadow']) && $display['box-shadow'] === true ) {
            $this->add_group_control(
                Group_Control_Box_Shadow::get_type(),
                [
                    'name'      => 'couponer_' . $name_section . '_box_shadow',
                    'label'     => esc_html__( 'Box Shadow', 'couponer-elementor' ),
                    'selector'  => '{{WRAPPER}} ' . $css_class,
                    'condition' => $condition,
                ]
            );
        }

    }

    /**
     *
     */
    public function get_post_category() {

        $wp_cat = get_categories('hide_empty=0');

        /** If no category is selected.  */
        $options = array();

        /** Add available post categories. */
        foreach( $wp_cat as $cat_item ){
            $options[$cat_item->slug] =  $cat_item->name;
        }

        return $options;
    }

    /**
     * @return array
     */
    public function get_post_types() {

        $wp_pt = get_post_types( array(
            'public' => 1,
        ) );
        $options = array();

        /** Add available post categories. */
        foreach( $wp_pt as $post_type ){
            $options[ $post_type ] = ucwords( $post_type );
        }

        return $options;
    }

    /**
     * @return array
     */
    public function get_post_taxonomies() {

        $taxonomies = get_taxonomies( array(
            'public'   => true,
            '_builtin' => false
        ) );

        /** Add default option */
        $options = array();

        foreach( $taxonomies as $taxonomy ) {

            $options[ $taxonomy ] = ucwords( $taxonomy );

        }

        return $options;

    }

    /**
     * @return array
     */
    public function get_post_taxonomy_terms() {

        $taxonomies = get_taxonomies( array(
            'public'   => true,
            '_builtin' => false
        ) );

        /** Add default option */
        $options = array();

        foreach( $taxonomies as $taxonomy ) {

            $terms = get_terms( $taxonomy, array(
                'hide_empty' => false,
            ) );

            foreach( $terms as $term ) {

                $options[ $term->slug ] = ucwords( $term->name );

            }

        }

        return $options;

    }

    /**
     * Add the widget controls.
     *
     * @since 1.0.0
     * @access protected
     *
     * @return void with category names
     **/
    protected function register_controls() {

        $this->tab_content();
        $this->tab_style();

    }

    /**
     * We return an array with the names of the layouts and their width.
     *
     * @param $settings - we get an array with all the data.
     *
     * @return array
     */
    public function get_LayoutSortable( $settings ) {

        /** We get all the data on the lists of layouts. */
        $layout = $settings['couponer_layout_priority'];

        /** An array in which the names of the layouts and their width will be stored. */
        $res = array();

        /** We write the names of the layouts and their width into an array. */
        foreach ($layout as $key => $val){
            $res[] = [ 'name' => $val['couponer_layout_item'] ];
        }

        return $res;

    }

    /**
     * Returns the size of the discount.
     *
     * @param $coupon - Coupon data.
     * @param $source - Custom data type or posts.
     *
     * @return string
     */
    public function get_coupon_discount( $coupon, $source ) {

        $discount_text = '';

        /** Define the text that should depend on the type of data. */
        if ( $source === 'custom' ) {
            $discount_text = $coupon['couponer_general_item_discount'];
        } elseif ( $source === 'posts' ) {
            $discount_text = get_post_meta( $coupon->ID, 'couponer_discount', true );
        }

        return '<div class="mdp-couponer-discount">' . esc_html( $discount_text ) . '</div>';
    }

    /**
     * Returns the coupon code.
     *
     * @param $coupon - Coupon data.
     * @param $source - Custom data type or posts.
     * @param $settings - Settings for displaying the coupon.
     */
    public function get_coupon_code( $coupon, $source, $settings ) {

        $coupon_style = $settings['couponer_general_coupon_style'];

        $code = '';
        $coupon_ID = '';

        if( $source === 'custom' && ( 'copy_code' === $coupon_style || 'expand' === $coupon_style ) ){
            $code_text = $coupon['couponer_general_item_code'];
            $coupon_ID = 'mdp-couponer-'.$coupon['_id'].'-'.$this->get_id();
            $id = trim($coupon['_id']);
        }elseif( $source === 'posts' && ( 'copy_code' === $coupon_style || 'expand' === $coupon_style ) ){
            $code_text = get_post_meta( $coupon->ID, 'couponer_code', true );
            $coupon_ID = 'mdp-couponer-'.$coupon->ID.'-'.$this->get_id();
            $id = trim($coupon->ID);
        }else{
            $code_text = '';
            $id = '';
        }

        /** Text for copying coupon code. */
        if( 'no_code' === $coupon_style ){
            $copy_text = $settings['couponer_general_no_code_text'];
            $class_text = 'mdp-couponer-no-code-text';
        }elseif( 'copy_code' === $coupon_style ){
            $copy_text = $settings['couponer_general_copy_code_text'];
            $class_text = 'mdp-couponer-copy-code-text';
        }elseif( 'expand' === $coupon_style ){
            $copy_text = $settings['couponer_general_expand_code_text'];
            $class_text = 'mdp-couponer-expand-text';
        }else{
            $copy_text = '';
            $class_text = '';
        }

        /** Icon for coupon code. */
        if( isset( $settings['couponer_general_icon_type'] ) && $settings['couponer_general_icon_type'] === 'icon'){
            $copy_icon = '<span class="mdp-couponer-code-icon"><i aria-hidden="true" class="fas ' . esc_attr($settings['couponer_general_icon']['value']) . '"></i></span>';
        } else {
            $copy_icon = '';
        }
        ?>

        <div id="<?php echo esc_attr( $coupon_ID ); ?>" class="mdp-couponer-code">

            <?php if( 'copy_code' === $coupon_style || 'expand' === $coupon_style ) : ?>

                <span id="mdp-couponer-code-<?php echo esc_attr( $id ); ?>"
                      class="mdp-couponer-text"><?php echo esc_html( $code_text ); ?></span>

            <?php endif; ?>

            <?php if ( $code_text !== '' ) : ?>

                <a href="javascript:void(0);" id="mpd-result-<?php echo esc_attr( $id ); ?>"
                   class="mdp-couponer-text"
                   onclick="copyCode('<?php echo esc_attr( $id ); ?>')"><?php echo $settings['couponer_general_copy_code_text'] ?></a>

            <?php endif; ?>

            <?php if( 'expand' === $coupon_style || 'no_code' === $coupon_style ) : ?>

                <span class="<?php echo esc_attr( $class_text ); ?>">

                    <?php echo  $copy_icon . esc_html( $copy_text ); ?>

                </span>

            <?php endif; ?>

        </div>

        <?php
    }

    /**
     * Return the image for the coupon.
     *
     * @param $coupon - Coupon data.
     * @param $source - Custom data type or posts.
     *
     */
    public function get_coupon_image( $coupon, $source ) {

        /** Define the image that should depend on the type of data. */
        if( $source === 'custom' ) {

            $image_url = $coupon['couponer_general_item_image']['url'];

        }elseif( $source === 'posts' ){

            $image_url = get_the_post_thumbnail_url( $coupon->ID, 'medium' );

            if( empty($image_url) ){
                $image_url =  UnityPlugin::get_url() . '/images/placeholder.png';
            }

        }else{
            $image_url = '';
        }

        ?>

        <div class="mdp-couponer-box-image">

            <img src="<?php echo esc_url($image_url); ?>"  class="mdp-couponer-image" alt="">

        </div>

        <?php

    }

    /**
     * Returns the title of the coupon.
     *
     * @param $coupon - Coupon data.
     * @param $source - Custom data type or posts.
     * @param $settings - Settings for displaying the coupon.
     *
     * @return string
     */
    public function get_coupon_title( $coupon, $source, $settings ) {

        /** Stores the coupon header code. */
        $title_text = '';

        /** Define the text that should depend on the type of data. */
        if( $source === 'custom' ){
            $title_text = $coupon['couponer_content_item_title'];
            $url = $coupon['couponer_link_item_link']['url'];
            $target = $coupon['couponer_link_item_link']['is_external'] ? ' target="_blank"' : '';
            $nofollow = $coupon['couponer_link_item_link']['nofollow'] ? ' rel="nofollow"' : '';
            $link_id = 'id=' . $coupon['couponer_link_id'];
        }elseif( $source === 'posts' ){
            $title_text = $coupon->post_title;
            $url = $coupon->guid;
            $link_id = '';
            $target = '';
            $nofollow = '';
        }else{
            $url = '';
            $link_id = '';
            $target = '';
            $nofollow = '';
        }

        $title = '<' . $settings['couponer_title_tag'] . ' class="mdp-couponer-title">';

        if( $settings['couponer_general_link_type'] === 'title' ){
            $title .= '<a href="' . esc_url( $url ) . '" ' . esc_attr( $link_id ) . ' '. $target . $nofollow .'>';
        }

        $title .= $title_text;

        if( $settings['couponer_general_link_type'] === 'title' ){
            $title .= '</a>';
        }

        $title .= '</' . $settings['couponer_title_tag'] . '>';

        return $title;
    }

    /**
     * Returns a brief description for the coupon.
     *
     * @param $coupon - Coupon data.
     * @param $source - Custom data type or posts.
     *
     * @return string
     */
    public function get_coupon_description( $coupon, $source ) {

        /** Contains brief description of the coupon. */
        $description_text = '';

        /** Define the text that should depend on the type of data. */
        if( $source === 'custom' ){
            $description_text = $coupon['couponer_content_item_description'];
        }elseif( $source === 'posts' ){
            $description_text =  $coupon->post_excerpt;
        }

        return '<p class="mdp-couponer-description">' . esc_html($description_text) . '</p>';

    }

    /**
     *  Returns the coupon link code.
     *
     * @param $coupon - Coupon data.
     * @param $source - Custom data type or posts.
     * @param $settings - Settings for displaying the coupon.
     *
     * @return string
     */
    public function get_coupon_link( $coupon, $source, $settings ) {

        /** Save link code to the html variable. */
        $link = '';

        /** We get the link for the coupon. */
        if( $source === 'custom' ){
            $url = $coupon['couponer_link_item_link']['url'];
            $target = $coupon['couponer_link_item_link']['is_external'] ? ' target="_blank"' : '';
            $nofollow = $coupon['couponer_link_item_link']['nofollow'] ? ' rel="nofollow"' : '';
            $link_id = $coupon['couponer_link_id'];
        }elseif( $source === 'posts' ){
            $url = $coupon->guid;
            $link_id = '';
            $target = '';
            $nofollow = '';
        }else{
            $url = '';
            $link_id = '';
            $target = '';
            $nofollow = '';
        }

        /** If the link is a button. */
        if ( $settings['couponer_general_link_type'] === 'button' ) {

            /** Start container button. */
            $link = '<div class="mdp-couponer-link-align">';
            $link .= '<div class="mdp-couponer-link"><a href="' . esc_url( $url ) . '" id="' . esc_attr( $link_id ) . '"  '. $target . $nofollow .'>';

            /** Display the button icon in before of the text. */
            if ( $settings['couponer_general_icon_position'] === 'before'  && $settings["couponer_general_button_icon"]['value'] !== null ) {
                $link .= '<span class="mdp-couponer-button-icon">';
                $link .= '<i aria-hidden="true" class="' . esc_attr__( $settings["couponer_general_button_icon"]['value'] ) . '"></i>';
                $link .= '</span>';
            }
            /** Button text. */
            $link .= esc_html( $settings['couponer_general_button_text'] );

            /** Display the button icon after the text. */
            if ( $settings['couponer_general_icon_position'] === 'after'  && $settings["couponer_general_button_icon"]['value'] !== null ) {
                $link .= '<span class="mdp-couponer-button-icon">';
                $link .= '<i aria-hidden="true" class="' . esc_attr__( $settings["couponer_general_button_icon"]['value'] ) . '"></i>';
                $link .= '</span>';
            }

            $link .= '</a></div>';
            $link .= '</div>';
            /** Button container end. */
        }

        return $link;
    }

    /**
     * @param $LayoutSortable
     * @param $coupon
     * @param $source
     */
    public function get_coupon( $LayoutSortable, $coupon, $source ) {

        /** Get all the values from the admin panel. */
        $settings = $this->get_settings_for_display();

        /** We get the link for the coupon. */
        switch ( $source ) {

            case 'custom':
                $url = $coupon['couponer_link_item_link']['url'];
                $target = !empty($coupon['couponer_link_item_link']['is_external']) ? ' target="_blank"' : '';
                $nofollow = $coupon['couponer_link_item_link']['nofollow'] ? ' rel="nofollow"' : '';
                $link_id = 'id="' . $coupon['couponer_link_id'].'"';
                $due_date = strtotime( $coupon[ 'couponer_general_due_date' ] );
                $expired = $due_date && $due_date < time();
                break;

            case 'posts':
                $url = $coupon->guid;
                $link_id = '';
                $target = '';
                $nofollow = '';
                $due_date_meta = get_post_meta( $coupon->ID, 'couponer_due_date', true );
                $due_date = strtotime( $due_date_meta );
                $expired = $due_date && $due_date < time();
                break;

            default:
                $url = '';
                $link_id = '';
                $target = '';
                $nofollow = '';
                $expired = false;

        }

        // Exit if coupon is expired
        if ( $expired ) { return; }

        echo '<div class="mdp-couponer-wrap">';

        echo '<div class="mdp-couponer-item">';

        /** We list the layouts in the given order. */
        foreach ( $LayoutSortable as $val ) {

            switch ( $val['name'] ) {
                case 'discount':
                    echo $this->get_coupon_discount( $coupon, $source );
                    break;
                case 'code':
                    $this->get_coupon_code( $coupon, $source, $settings );
                    break;
                case 'image':
                    $this->get_coupon_image( $coupon, $source );
                    break;
                case 'title':
                    echo $this->get_coupon_title( $coupon, $source, $settings );
                    break;
                case 'description':
                    echo $this->get_coupon_description( $coupon, $source );
                    break;
                case 'link':
                    echo $this->get_coupon_link( $coupon, $source, $settings );
                    break;
                case 'custom':
                    $custom_text = '';
                    if ( is_array( $settings[ 'couponer_layout_priority' ] ) ) {
                        foreach ( $settings[ 'couponer_layout_priority' ] as &$value ) {
                            if ( is_array( $value ) && isset( $value[ 'couponer_layout_custom_text' ] ) ) {
                                $custom_text = $value[ 'couponer_layout_custom_text' ];
                            }
                        }
                    }
                    echo esc_html( $custom_text );
                    break;
                default:
                    echo "You have not added any items to the layout.";
            }

        }

        /** The link is stretched to the entire container. */
        if( isset( $settings['couponer_general_link_type'] ) && $settings['couponer_general_link_type'] === 'box' ){
            echo '<a href="' . esc_url( $url ) . '" id="' . esc_attr( $link_id ) . '" '. $target . $nofollow .' class="mdp-couponer-box-link"></a>';
        }

        echo '</div>';

        echo '</div>';

    }

    /**
     * Render Frontend Output. Generate the final HTML on the frontend.
     *
     * @since 1.0.0
     * @access protected
     **/
    protected function render() {

        /** Get all the values from the admin panel. */
        $settings = $this->get_settings_for_display();

        /** Unique widget identifier. */
        $idItem = $this->get_id();

        /** We return an array with the names of the layouts and their width. */
        $LayoutSortable = $this->get_LayoutSortable( $settings );

        echo '<div class="mdp-couponer-container mdp-couponer-container-'.esc_attr__($idItem).'">';

        if( $settings['couponer_general_source'] === 'custom' ){

            /** Count the number of coupons. */
            $quantity_coupons = count( $settings['couponer_layout_coupon'] ) - 1;

            /** We display coupons in a cycle. */
            for ( $i = 0; $i <= $quantity_coupons; $i++){
                $this->get_coupon( $LayoutSortable, $settings['couponer_layout_coupon'][$i], 'custom' );
            }

        } elseif ( $settings['couponer_general_source'] === 'posts' ) {

            $args = array(
                'numberposts' => 50,
                'posts_per_page' => 0,
                'orderby'       => 'date',
                'order'         => 'DESC'
            );

            /** We get an array with post data. */
            if ( $settings['post_type'] === 'post' ) {

                $args = array_merge( $args, array(
                    'post_type'     => 'post',
                    'category_name' => $settings[ 'select_category' ],
                ) );

            } else {

                $args = array_merge( $args, array(
                    'post_type'   => array( $settings[ 'post_type' ] ),
                    'category_name' => '',
                ) );

                // Add taxonomies related args
                if ( $settings[ 'post_taxonomy' ] !== '' ) {

                    $args = array_merge( $args,
                        array(
                            'tax_query' => array(
                                array(
                                    'taxonomy' => $settings[ 'post_taxonomy' ],
                                    'field' => 'slug',
                                    'terms' => $settings[ 'post_taxonomy_terms' ],
                                    'operator' => 'IN',
                                )
                            )
                        )
                    );

                }

            }

            /** Get posts array */
            $posts = get_posts( $args );

            /** We count the number of coupons in the category. */
            $quantity_posts_coupons = count( $posts ) - 1;

            /** We display coupons in a cycle. */
            for ( $i = 0; $i <= $quantity_posts_coupons; $i++){
                $this->get_coupon( $LayoutSortable, $posts[$i], 'posts' );
            }

        }

        echo '</div>';

    }

    /**
     * Return link for documentation
     * Used to add stuff after widget
     *
     * @since 1.0.0
     * @access public
     **/
    public function get_custom_help_url() {
        return 'https://docs.merkulov.design/tag/couponer/';
    }

    /**
     * Register controls for the Content Tab
     *
     * @since 1.0.0
     * @access private
     */
    private function tab_content() {
        $this->section_general();
        $this->section_layout();
        $this->section_coupons();
    }

    /**
     * Register controls for the Style Tab
     *
     * @since 1.0.0
     * @access private
     */
    private function tab_style() {
        $this->section_grid();
        $this->section_discount();
        $this->section_code();
        $this->section_image();
        $this->section_title();
        $this->section_description();
        $this->section_button();
    }

    /**
     * General section -> Content Tab
     *
     * @since 1.0.0
     * @access private
     */
    private function section_general() {

        /** General */
        $this->start_controls_section(
            'couponer_general',
            [
                'label' => esc_html__( 'General', 'couponer-elementor' ),
                'tab'   => Controls_Manager::TAB_CONTENT,
            ]
        );

        /** Source. */
        $this->add_control(
            'couponer_general_source',
            [
                'label'   => esc_html__( 'Source', 'couponer-elementor' ),
                'type'    => Controls_Manager::SELECT,
                'default' => 'custom',
                'options' => [
                    'custom' => esc_html__( 'Custom', 'couponer-elementor' ),
                    'posts'  => esc_html__( 'Posts', 'couponer-elementor' ),
                ],
            ]
        );

        $this->add_control(
            'post_type',
            [
                'label'     => esc_html__( 'Post Type', 'couponer-elementor' ),
                'type'      => Controls_Manager::SELECT,
                'label_block' => true,
                'default'   => 'post',
                'options'   => $this->get_post_types(),
                'condition' => [ 'couponer_general_source' => 'posts' ]
            ]
        );

        $this->add_control(
            'select_category',
            [
                'label'     => esc_html__( 'Category', 'couponer-elementor' ),
                'type'      => Controls_Manager::SELECT,
                'label_block' => true,
                'options'   => $this->get_post_category(),
                'conditions' => [
                    'terms' => [
                        [
                            'name' => 'post_type',
                            'operator' => 'in',
                            'value' => [
                                'post',
                            ],
                        ],
                        [
                            'name' => 'couponer_general_source',
                            'operator' => 'in',
                            'value' => [
                                'posts',
                            ],
                        ],
                    ],
                ],

            ]
        );

        $this->add_control(
            'post_taxonomy',
            [
                'label'     => esc_html__( 'Taxonomy', 'couponer-elementor' ),
                'type'      => Controls_Manager::SELECT2,
                'label_block' => true,
                'options'   => $this->get_post_taxonomies(),
                'condition' => [ 'couponer_general_source' => 'posts' ]
            ]
        );

        $this->add_control(
            'post_taxonomy_terms',
            [
                'label'     => esc_html__( 'Taxonomy terms', 'couponer-elementor' ),
                'type'      => Controls_Manager::SELECT2,
                'label_block' => true,
                'multiple' => true,
                'options'   => $this->get_post_taxonomy_terms(),
                'condition' => [ 'couponer_general_source' => 'posts' ]
            ]
        );

        /** Columns. */
        $this->add_responsive_control(
            'couponer_general_columns',
            [
                'label'           => esc_html__( 'Columns', 'couponer-elementor' ),
                'type'            => Controls_Manager::SLIDER,
                'size_units'      => [ 'px' ],
                'devices'         => [ 'desktop', 'tablet', 'mobile' ],
                'range'           => [
                    'px' => [
                        'min'  => 1,
                        'max'  => 10,
                        'step' => 1,
                    ],
                ],
                'desktop_default' => [
                    'unit' => 'px',
                    'size' => 2,
                ],
                'selectors'       => [
                    '{{WRAPPER}} .mdp-couponer-wrap' => 'width: calc(100% / {{size}});',
                ],
            ]
        );

        /** Coupon Style. */
        $this->add_control(
            'couponer_general_coupon_style',
            [
                'label'   => esc_html__( 'Coupon Style', 'couponer-elementor' ),
                'type'    => Controls_Manager::SELECT,
                'default' => 'no_code',
                'options' => [
                    'no_code'   => esc_html__( 'No Code', 'couponer-elementor' ),
                    'copy_code' => esc_html__( 'Copy Code', 'couponer-elementor' ),
                    'expand'    => esc_html__( 'Expand Code and Copy', 'couponer-elementor' ),
                ],
            ]
        );

        /** No Code Text */
        $this->add_control(
            'couponer_general_no_code_text',
            [
                'label'       => esc_html__( 'No Code Message', 'couponer-elementor' ),
                'type'        => Controls_Manager::TEXT,
                'dynamic'     => [ 'active' => true ],
                'default'     => esc_html__( 'No Code', 'couponer-elementor' ),
                'placeholder' => esc_html__( 'Message', 'couponer-elementor' ),
                'condition'   => [ 'couponer_general_coupon_style' => 'no_code' ]
            ]
        );

        /** Copy Code Text. */
        $this->add_control(
            'couponer_general_copy_code_text',
            [
                'label'       => esc_html__( 'Copy Text', 'couponer-elementor' ),
                'type'        => Controls_Manager::TEXT,
                'dynamic'     => [ 'active' => true ],
                'default'     => esc_html__( 'Copy', 'couponer-elementor' ),
                'placeholder' => esc_html__( 'Copy', 'couponer-elementor' ),
                'condition'   => [ 'couponer_general_coupon_style!' => 'no_code' ]
            ]
        );

        /** Expand Code Text. */
        $this->add_control(
            'couponer_general_expand_code_text',
            [
                'label'       => esc_html__( 'Expand Code Message', 'couponer-elementor' ),
                'type'        => Controls_Manager::TEXT,
                'dynamic'     => [ 'active' => true ],
                'default'     => esc_html__( 'Expand', 'couponer-elementor' ),
                'placeholder' => esc_html__( 'Expand', 'couponer-elementor' ),
                'condition'   => [ 'couponer_general_coupon_style' => 'expand' ]
            ]
        );

        /** Coupon Icon. */
        $this->add_control(
            'couponer_general_icon_type',
            [
                'label'       => esc_html__( 'Coupon Icon', 'couponer-elementor' ),
                'type'        => Controls_Manager::CHOOSE,
                'label_block' => false,
                'toggle'      => false,
                'options'     => [
                    'none' => [
                        'title' => esc_html__( 'None', 'couponer-elementor' ),
                        'icon'  => 'fa fa-ban',
                    ],
                    'icon' => [
                        'title' => esc_html__( 'Icon', 'couponer-elementor' ),
                        'icon'  => 'fa fa-flag',
                    ]
                ],
                'default'     => 'icon',
                'condition'   => [ 'couponer_general_coupon_style!' => 'copy_code' ]
            ]
        );

        /** Choose Icon */
        $this->add_control(
            'couponer_general_icon',
            [
                'label'            => esc_html__( 'Choose Icon', 'couponer-elementor' ),
                'type'             => Controls_Manager::ICONS,
                'fa4compatibility' => 'icon',
                'default'          => [
                    'value'   => 'fas fa-bell',
                    'library' => 'fa-solid',
                ],
                'condition'        => [
                    'couponer_general_icon_type'     => 'icon',
                    'couponer_general_coupon_style!' => 'copy_code'
                ]
            ]
        );

        /** Link. */
        $this->add_control(
            'couponer_general_header_link',
            [
                'label'     => __( 'Link', 'couponer-elementor' ),
                'type'      => Controls_Manager::HEADING,
                'separator' => 'before',
            ]
        );

        /** Link Type. */
        $this->add_control(
            'couponer_general_link_type',
            [
                'label'   => esc_html__( 'Link Type', 'couponer-elementor' ),
                'type'    => Controls_Manager::SELECT,
                'default' => 'none',
                'options' => [
                    'none'   => esc_html__( 'None', 'couponer-elementor' ),
                    'box'    => esc_html__( 'Box', 'couponer-elementor' ),
                    'title'  => esc_html__( 'Title', 'couponer-elementor' ),
                    'button' => esc_html__( 'Button', 'couponer-elementor' ),
                ],
            ]
        );

        /** Button Text. */
        $this->add_control(
            'couponer_general_button_text',
            [
                'label'       => esc_html__( 'Button Text', 'couponer-elementor' ),
                'type'        => Controls_Manager::TEXT,
                'dynamic'     => [ 'active' => true ],
                'default'     => esc_html__( 'Button Text', 'couponer-elementor' ),
                'placeholder' => esc_html__( 'Button Text', 'couponer-elementor' ),
                'condition'   => [ 'couponer_general_link_type' => 'button' ]
            ]
        );

        /** Button Icon */
        $this->add_control(
            'couponer_general_button_icon',
            [
                'label'            => esc_html__( 'Button Icon', 'couponer-elementor' ),
                'type'             => Controls_Manager::ICONS,
                'fa4compatibility' => 'icon',
                'default'          => [
                    'value'   => 'fas fa-bell',
                    'library' => 'fa-solid',
                ],
                'condition'        => [ 'couponer_general_link_type' => 'button' ]
            ]
        );

        /** Icon Position. */
        $this->add_control(
            'couponer_general_icon_position',
            [
                'label'     => esc_html__( 'Icon Position', 'couponer-elementor' ),
                'type'      => Controls_Manager::SELECT,
                'default'   => 'before',
                'options'   => [
                    'before' => esc_html__( 'Before', 'couponer-elementor' ),
                    'after'  => esc_html__( 'After', 'couponer-elementor' ),
                ],
                'condition' => [ 'couponer_general_link_type' => 'button' ]
            ]
        );

        $this->end_controls_section();

    }

    /**
     * Layout section -> Content Tab
     *
     * @since 1.0.0
     * @access private
     */
    private function section_layout() {

        /** Layout */
        $this->start_controls_section(
            'couponer_layout',
            [
                'label' => esc_html__( 'Layout', 'couponer-elementor' ),
                'tab'   => Controls_Manager::TAB_CONTENT,
            ]
        );

        $this->add_responsive_control(
            'couponer_box_align',
            [
                'label'     => esc_html__( 'Alignment', 'couponer-elementor' ),
                'type'      => Controls_Manager::CHOOSE,
                'options'   => [
                    'start'         => [
                        'title' => esc_html__( 'Left', 'couponer-elementor' ),
                        'icon'  => 'fa fa-align-left',
                    ],
                    'center'        => [
                        'title' => esc_html__( 'Center', 'couponer-elementor' ),
                        'icon'  => 'fa fa-align-center',
                    ],
                    'end'           => [
                        'title' => esc_html__( 'Right', 'couponer-elementor' ),
                        'icon'  => 'fa fa-align-right',
                    ],
                    'space-between' => [
                        'title' => esc_html__( 'Space between', 'couponer-elementor' ),
                        'icon'  => 'fa fa-expand-arrows-alt',
                    ],
                    'space-around'  => [
                        'title' => esc_html__( 'Space around', 'couponer-elementor' ),
                        'icon'  => 'fa fa-compress-arrows-alt',
                    ],
                ],
                'default'   => 'space-around',
                'selectors' => [
                    '{{WRAPPER}} .mdp-couponer-item' => 'justify-content: {{value}};',
                ],
                'toggle'    => true,
            ]
        );

        $layout = new Repeater();

        /** Item. */
        $layout->add_control(
            'couponer_layout_item',
            [
                'label'   => esc_html__( 'Item', 'couponer-elementor' ),
                'type'    => Controls_Manager::SELECT,
                'default' => 'title',
                'options' => [
                    'discount'    => esc_html__( 'Discount', 'couponer-elementor' ),
                    'code'        => esc_html__( 'Code', 'couponer-elementor' ),
                    'image'       => esc_html__( 'Image', 'couponer-elementor' ),
                    'title'       => esc_html__( 'Title', 'couponer-elementor' ),
                    'description' => esc_html__( 'Description', 'couponer-elementor' ),
                    'link'        => esc_html__( 'Button', 'couponer-elementor' ),
                    'custom'      => esc_html__( 'Custom Text', 'couponer-elementor' ),
                ],
            ]
        );

        $layout->add_control(
            'couponer_layout_custom_text',
            [
                'label'       => esc_html__( 'Custom Text', 'couponer-elementor' ),
                'type'        => Controls_Manager::TEXT,
                'dynamic'     => [ 'active' => true ],
                'label_block' => true,
                'default'     => esc_html__( 'Custom Text', 'couponer-elementor' ),
                'placeholder' => esc_html__( 'Custom Text', 'couponer-elementor' ),
                'condition'   => [ 'couponer_layout_item' => 'custom' ]
            ]
        );

        /** Priority. */
        $this->add_control(
            'couponer_layout_priority',
            [
                'label'       => esc_html__( 'Priority', 'couponer-elementor' ),
                'type'        => Controls_Manager::REPEATER,
                'fields'      => $layout->get_controls(),
                'default'     => [
                    [ 'couponer_layout_item' => 'discount' ],
                    [ 'couponer_layout_item' => 'code' ],
                    [ 'couponer_layout_item' => 'image' ],
                    [ 'couponer_layout_item' => 'title' ],
                ],
                'separator'   => 'before',
                'title_field' => 'Layout item  - {{{couponer_layout_item}}}'
            ]
        );

        $this->end_controls_section();

    }

    /**
     * Coupons content section -> Content Tab
     *
     * @since 1.0.0
     * @access private
     */
    private function section_coupons() {

        /** Coupons */
        $this->start_controls_section(
            'couponer_coupons',
            [
                'label'     => esc_html__( 'Coupons', 'couponer-elementor' ),
                'tab'       => Controls_Manager::TAB_CONTENT,
                'condition' => [ 'couponer_general_source' => 'custom' ]
            ]
        );

        $coupons = new Repeater();

        $coupons->start_controls_tabs( 'tabs-couponer' );

        $coupons->start_controls_tab( 'tab_general', [ 'label' => esc_html__( 'General', 'tabs-general' ) ] );

        /** Choose Image. */
        $coupons->add_control(
            'couponer_general_item_image',
            [
                'label'   => __( 'Choose Image', 'couponer-elementor' ),
                'type'    => Controls_Manager::MEDIA,
                'default' => [
                    'url' => Utils::get_placeholder_image_src(),
                ],
            ]
        );

        /** Discount. */
        $coupons->add_control(
            'couponer_general_item_discount',
            [
                'label'       => esc_html__( 'Discount description', 'couponer-elementor' ),
                'type'        => Controls_Manager::TEXT,
                'dynamic'     => [ 'active' => true ],
                'label_block' => true,
                'default'     => esc_html__( 'Discount', 'couponer-elementor' ),
                'placeholder' => esc_html__( 'Discount', 'couponer-elementor' ),
            ]
        );

        /** Coupon Code */
        $coupons->add_control(
            'couponer_general_item_code',
            [
                'label'       => esc_html__( 'Coupon Code', 'couponer-elementor' ),
                'type'        => Controls_Manager::TEXT,
                'dynamic'     => [ 'active' => true ],
                'label_block' => true,
                'default'     => esc_html__( 'Coupon Code', 'couponer-elementor' ),
                'placeholder' => esc_html__( 'Coupon Code', 'couponer-elementor' ),
            ]
        );

        /** Coupons */
        $coupons->add_control(
            'couponer_general_due_date',
            [
                'label' => esc_html__( 'Due Date', 'couponer-elementor' ),
                'type' => Controls_Manager::DATE_TIME,
                'description' => esc_html__( 'Due date related to the UNIX date. Current UNIX date: ', 'couponer-elementor' ) . '<b>' . date('Y-m-d H:i', time() ) . '</b>',
                'label_block' => true
            ]
        );

        $coupons->end_controls_tab();

        $coupons->start_controls_tab( 'tab_content', [ 'label' => esc_html__( 'Content', 'tabs-content' ) ] );

        /** Title. */
        $coupons->add_control(
            'couponer_content_item_title',
            [
                'label'       => esc_html__( 'Title', 'couponer-elementor' ),
                'type'        => Controls_Manager::TEXT,
                'dynamic'     => [ 'active' => true ],
                'default'     => esc_html__( 'Title', 'couponer-elementor' ),
                'placeholder' => esc_html__( 'Title', 'couponer-elementor' ),
            ]
        );

        /** Description. */
        $coupons->add_control(
            'couponer_content_item_description',
            [
                'label'       => esc_html__( 'Description', 'couponer-elementor' ),
                'type'        => Controls_Manager::TEXTAREA,
                'rows'        => 3,
                'default'     => esc_html__( 'Description', 'couponer-elementor' ),
                'placeholder' => esc_html__( 'Description', 'couponer-elementor' ),
            ]
        );

        $coupons->end_controls_tab();

        $coupons->start_controls_tab( 'tab_link', [ 'label' => esc_html__( 'Link', 'tabs-link' ) ] );

        /** Link. */
        $coupons->add_control(
            'couponer_link_item_link',
            [
                'label'         => esc_html__( 'Link', 'couponer-elementor' ),
                'type'          => Controls_Manager::URL,
                'placeholder'   => esc_html__( 'https://codecanyon.net/user/merkulove', 'couponer-elementor' ),
                'show_external' => true,
                'default'       => [
                    'url'         => esc_url( '#' ),
                    'is_external' => true,
                    'nofollow'    => true,
                ],
            ]
        );

        /** Link id. */
        $coupons->add_control(
            'couponer_link_id',
            [
                'label'       => esc_html__( 'Link id', 'couponer-elementor' ),
                'type'        => Controls_Manager::TEXT,
                'dynamic'     => [ 'active' => true ],
                'placeholder' => esc_html__( 'Link id', 'couponer-elementor' ),
            ]
        );

        $coupons->end_controls_tab();

        $coupons->end_controls_tabs();

        /** Coupons. */
        $this->add_control(
            'couponer_layout_coupon',
            [
                'label'       => esc_html__( 'Coupon', 'couponer-elementor' ),
                'type'        => Controls_Manager::REPEATER,
                'fields'      => $coupons->get_controls(),
                'default'     => [
                    [
                        'couponer_general_item_image'       => '',
                        'couponer_general_item_discount'    => '15% OFF',
                        'couponer_general_item_code'        => '#mlove15',
                        'couponer_content_item_title'       => 'Coupon Title 1',
                        'couponer_content_item_description' => 'Lorem Ipsum is simply dummy text of the printing and typesetting industry.',
                        'couponer_link_item_link'           => '#',
                        'couponer_link_id'                  => 'mlove1'
                    ],
                    [
                        'couponer_general_item_image'       => '',
                        'couponer_general_item_discount'    => '10% OFF',
                        'couponer_general_item_code'        => '#mlove10',
                        'couponer_content_item_title'       => 'Coupon Title 2',
                        'couponer_content_item_description' => 'Lorem Ipsum is simply dummy text of the printing and typesetting industry.',
                        'couponer_link_item_link'           => '#',
                        'couponer_link_id'                  => 'mlove2'
                    ],
                    [
                        'couponer_general_item_image'       => '',
                        'couponer_general_item_discount'    => '25% OFF',
                        'couponer_general_item_code'        => '#mlove25',
                        'couponer_content_item_title'       => 'Coupon Title 3',
                        'couponer_content_item_description' => 'Lorem Ipsum is simply dummy text of the printing and typesetting industry.',
                        'couponer_link_item_link'           => '#',
                        'couponer_link_id'                  => 'mlove3'
                    ],
                    [
                        'couponer_general_item_image'       => '',
                        'couponer_general_item_discount'    => '20% OFF',
                        'couponer_general_item_code'        => '#mlove20',
                        'couponer_content_item_title'       => 'Coupon Title 4',
                        'couponer_content_item_description' => 'Lorem Ipsum is simply dummy text of the printing and typesetting industry.',
                        'couponer_link_item_link'           => '#',
                        'couponer_link_id'                  => 'mlove4'
                    ],
                ],
                'separator'   => 'before',
                'title_field' => 'Coupon  - {{{couponer_content_item_title}}}'
            ]
        );

        $this->end_controls_section();

    }

    /**
     * Grid section -> Style Tab
     *
     * @since 1.0.0
     * @access private
     */
    private function section_grid() {

        /** Grid Boxes. */
        $this->start_controls_section( 'style_grid_boxes',
            [
                'label' => esc_html__( 'Grid Boxes', 'couponer-elementor' ),
                'tab'   => Controls_Manager::TAB_STYLE,
            ] );

        /** Column spacing. */
        $this->add_responsive_control(
            'couponer_grid_column_spacing',
            [
                'label'      => esc_html__( 'Column spacing', 'couponer-elementor' ),
                'type'       => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'devices'    => [ 'desktop', 'tablet', 'mobile' ],
                'range'      => [
                    'px' => [
                        'min'  => 1,
                        'step' => 1,
                    ],
                ],
                'default'    => [
                    'unit' => 'px',
                    'size' => 30,
                ],
                'selectors'  => [
                    '{{WRAPPER}} .mdp-couponer-wrap'      => 'padding-left: calc({{size}}{{unit}} / 2); padding-right: calc({{size}}{{unit}} / 2);',
                    '{{WRAPPER}} .mdp-couponer-container' => 'margin-left: calc(-{{size}}{{unit}} / 2); margin-right: calc(-{{size}}{{unit}} / 2);',
                ],
            ]
        );

        /** Row spacing. */
        $this->add_responsive_control(
            'couponer_grid_row_spacing',
            [
                'label'      => esc_html__( 'Row spacing', 'couponer-elementor' ),
                'type'       => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'devices'    => [ 'desktop', 'tablet', 'mobile' ],
                'range'      => [
                    'px' => [
                        'min'  => 1,
                        'step' => 1,
                    ],
                ],
                'default'    => [
                    'unit' => 'px',
                    'size' => 30,
                ],
                'selectors'  => [
                    '{{WRAPPER}} .mdp-couponer-wrap' => 'margin-bottom: {{size}}{{unit}};',
                ],
                'separator'  => 'after'
            ]
        );

        $this->get_settings_group(
            'grid',
            '.mdp-couponer-item',
            [],
            [
                'margin'        => true,
                'padding'       => true,
                'color'         => true,
                'typography'    => true,
                'text-shadow'   => true,
                'alignment'     => true,
                'background'    => true,
                'border'        => true,
                'border-radius' => true,
                'box-shadow'    => true,
            ]
        );

        $this->end_controls_section();

    }

    /**
     * Discount section -> Style Tab
     *
     * @since 1.0.0
     * @access private
     */
    private function section_discount() {

        /** Discount. */
        $this->start_controls_section( 'style_discount',
            [
                'label' => esc_html__( 'Discount', 'couponer-elementor' ),
                'tab'   => Controls_Manager::TAB_STYLE,
            ] );

        $this->get_settings_group(
            'discount',
            '.mdp-couponer-discount',
            [],
            [
                'width'         => true,
                'margin'        => true,
                'padding'       => true,
                'color'         => true,
                'typography'    => true,
                'text-shadow'   => true,
                'alignment'     => true,
                'background'    => true,
                'border'        => true,
                'border-radius' => true,
                'box-shadow'    => true,
            ]
        );

        $this->end_controls_section();

    }

    /**
     * Coupon Code section -> Style Tab
     *
     * @since 1.0.0
     * @access private
     */
    private function section_code() {

        $this->start_controls_section( 'style_code',
            [
                'label' => esc_html__( 'Code', 'couponer-elementor' ),
                'tab'   => Controls_Manager::TAB_STYLE,
            ] );

        $this->controls_code();
        $this->controls_nocode();
        $this->controls_copycode();
        $this->controls_expandcode();
        $this->controls_codeicon();

        $this->end_controls_section();

    }

    /**
     * Coupon Image section -> Style Tab
     *
     * @since 1.0.0
     * @access private
     */
    private function section_image() {

        /** Image. */
        $this->start_controls_section( 'style_image',
            [
                'label' => esc_html__( 'Image', 'couponer-elementor' ),
                'tab'   => Controls_Manager::TAB_STYLE,
            ] );

        /** Margin. */
        $this->add_responsive_control(
            'couponer_image_margin',
            [
                'label'      => esc_html__( 'Margin', 'couponer-elementor' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors'  => [
                    '{{WRAPPER}} .mdp-couponer-box-image' => 'margin: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
                ],
                'toggle'     => true,
            ]
        );

        /** Padding. */
        $this->add_responsive_control(
            'couponer_image_padding',
            [
                'label'      => esc_html__( 'Padding', 'couponer-elementor' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors'  => [
                    '{{WRAPPER}} .mdp-couponer-box-image' => 'padding: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
                ],
                'toggle'     => true,
                'separator'  => 'after'
            ]
        );

        /** Background. */
        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name'      => 'couponer_image_background',
                'label'     => esc_html__( 'Background', 'couponer-elementor' ),
                'types'     => [ 'classic', 'gradient' ],
                'separator' => 'before',
                'selector'  => '{{WRAPPER}} .mdp-couponer-box-image',
            ]
        );

        /** Border. */
        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name'      => 'couponer_image_border',
                'label'     => esc_html__( 'Border', 'couponer-elementor' ),
                'selector'  => '{{WRAPPER}} .mdp-couponer-box-image img',
                'separator' => 'before',
            ]
        );

        /** Border Radius. */
        $this->add_responsive_control(
            'couponer_image_border_radius',
            [
                'label'      => esc_html__( 'Border Radius', 'couponer-elementor' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors'  => [
                    '{{WRAPPER}} .mdp-couponer-box-image img' => 'border-radius: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
                ],
                'separator'  => 'after',
                'toggle'     => true,
            ]
        );

        /** Box Shadow. */
        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name'     => 'couponer_image_box_shadow',
                'label'    => esc_html__( 'Box Shadow', 'couponer-elementor' ),
                'selector' => '{{WRAPPER}} .mdp-couponer-box-image',
            ]
        );

        /** CSS filter. */
        $this->add_group_control(
            Group_Control_Css_Filter::get_type(),
            [
                'name'     => 'couponer_css_filter',
                'label'    => esc_html__( 'CSS filter', 'couponer-elementor' ),
                'selector' => '{{WRAPPER}} .mdp-couponer-image',
            ]
        );

        /** Image Sizes */
        $this->add_control(
            'header_image_sizes',
            [
                'label'     => esc_html__( 'Image Sizes', 'couponer-elementor' ),
                'type'      => Controls_Manager::HEADING,
                'separator' => 'before',
            ]
        );

        /** Width. */
        $this->add_responsive_control(
            'couponer_image_sizes_width',
            [
                'label'      => esc_html__( 'Width', 'couponer-elementor' ),
                'type'       => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'devices'    => [ 'desktop', 'tablet', 'mobile' ],
                'range'      => [
                    'px' => [
                        'min'  => 1,
                        'step' => 1,
                    ],
                ],
                'selectors'  => [
                    '{{WRAPPER}} .mdp-couponer-image' => 'width: {{size}}{{unit}}',

                ],
            ]
        );

        /** Height. */
        $this->add_responsive_control(
            'couponer_image_sizes_height',
            [
                'label'      => esc_html__( 'Height', 'couponer-elementor' ),
                'type'       => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'devices'    => [ 'desktop', 'tablet', 'mobile' ],
                'range'      => [
                    'px' => [
                        'min'  => 1,
                        'step' => 1,
                    ],
                ],
                'selectors'  => [
                    '{{WRAPPER}} .mdp-couponer-image' => 'height: {{size}}{{unit}};',
                ],
                'separator'  => 'after'
            ]
        );

        $this->end_controls_section();

    }

    /**
     * Coupon Title section -> Style Tab
     *
     * @since 1.0.0
     * @access private
     */
    private function section_title() {

        /** Title. */
        $this->start_controls_section( 'style_title',
            [
                'label' => esc_html__( 'Title', 'couponer-elementor' ),
                'tab'   => Controls_Manager::TAB_STYLE,
            ] );

        /** HTML Tag. */
        $this->add_control(
            'couponer_title_tag',
            [
                'label'   => esc_html__( 'HTML Tag', 'couponer-elementor' ),
                'type'    => Controls_Manager::SELECT,
                'options' => [
                    'h1'   => 'H1',
                    'h2'   => 'H2',
                    'h3'   => 'H3',
                    'h4'   => 'H4',
                    'h5'   => 'H5',
                    'h6'   => 'H6',
                    'div'  => 'div',
                    'span' => 'span',
                    'p'    => 'p',
                ],
                'default' => 'div',
            ]
        );

        $this->get_settings_group(
            'title',
            '.mdp-couponer-title',
            [],
            [
                'width'         => true,
                'margin'        => true,
                'padding'       => true,
                'color'         => true,
                'typography'    => true,
                'text-shadow'   => true,
                'alignment'     => true,
                'background'    => true,
                'border'        => true,
                'border-radius' => true,
                'box-shadow'    => true,
            ]
        );

        $this->end_controls_section();

    }

    /**
     * Coupon Description section -> Style Tab
     *
     * @since 1.0.0
     * @access private
     */
    private function section_description() {

        /** Description. */
        $this->start_controls_section( 'style_description',
            [
                'label' => esc_html__( 'Description', 'couponer-elementor' ),
                'tab'   => Controls_Manager::TAB_STYLE,
            ] );

        $this->get_settings_group(
            'description',
            '.mdp-couponer-description',
            [],
            [
                'width'         => true,
                'margin'        => true,
                'padding'       => true,
                'color'         => true,
                'typography'    => true,
                'text-shadow'   => true,
                'alignment'     => true,
                'background'    => true,
                'border'        => true,
                'border-radius' => true,
                'box-shadow'    => true,
            ]
        );

        $this->end_controls_section();

    }

    /**
     * Coupon Button section -> Style Tab
     *
     * @since 1.0.0
     * @access private
     */
    private function section_button() {

        /** Button. */
        $this->start_controls_section( 'style_button',
            [
                'label' => esc_html__( 'Button', 'couponer-elementor' ),
                'tab'   => Controls_Manager::TAB_STYLE,
            ] );

        $this->add_responsive_control(
            'couponer_button_width',
            [
                'label'      => esc_html__( 'Width', 'couponer-elementor' ),
                'type'       => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'default'    => [
                    'unit' => '%',
                    'size' => 100,
                ],
                'range'      => [
                    '%' => [
                        'min'  => 0,
                        'max'  => 100,
                        'step' => 1,
                    ],
                ],
                'selectors'  => [
                    '{{WRAPPER}} .mdp-couponer-link-align' => 'width: {{size}}{{unit}};',
                ],
                'separator'  => 'after'
            ]
        );

        $this->controls_tabs_button();

        $this->end_controls_section();

    }

    /**
     * Register Controls for Coupon Code
     *
     * @since 1.0.0
     * @access private
     */
    private function controls_code() {

        $this->add_control(
            'header_code_text',
            [
                'label'     => esc_html__( 'Code', 'couponer-elementor' ),
                'type'      => Controls_Manager::HEADING,
                'condition' => [ 'couponer_general_coupon_style!' => 'no_code' ],
            ]
        );

        $this->get_settings_group(
            'code',
            '.mdp-couponer-code',
            [ 'couponer_general_coupon_style!' => 'no_code' ],
            [
                'width'         => true,
                'margin'        => true,
                'padding'       => true,
                'color'         => true,
                'typography'    => true,
                'text-shadow'   => true,
                'alignment'     => true,
                'background'    => true,
                'border'        => true,
                'border-radius' => true,
                'box-shadow'    => true,
            ]
        );
    }

    /**
     * Register Controls for Coupon No Code Message
     *
     * @since 1.0.0
     * @access private
     */
    private function controls_nocode() {

        $this->add_control(
            'header_nocode_text',
            [
                'label'     => esc_html__( 'No Code Message', 'couponer-elementor' ),
                'type'      => Controls_Manager::HEADING,
                'separator' => 'before',
                'condition' => [ 'couponer_general_coupon_style' => 'no_code' ]
            ]
        );

        /** No code text. */
        $this->get_settings_group(
            'nocodetext',
            '.mdp-couponer-code',
            [ 'couponer_general_coupon_style' => 'no_code' ],
            [
                'width'         => true,
                'margin'        => true,
                'padding'       => true,
                'color'         => true,
                'typography'    => true,
                'text-shadow'   => true,
                'alignment'     => true,
                'background'    => true,
                'border'        => true,
                'border-radius' => true,
                'box-shadow'    => true,
            ]
        );

    }

    /**
     * Register Controls for Coupon Copy Code Button
     *
     * @since 1.0.0
     * @access private
     */
    private function controls_copycode() {

        $this->add_control(
            'header_copycode_text',
            [
                'label'     => esc_html__( 'Copy Button', 'couponer-elementor' ),
                'type'      => Controls_Manager::HEADING,
                'separator' => 'before',
                'condition' => [ 'couponer_general_coupon_style' => 'copy_code' ]
            ]
        );

        /** Copy code text. */
        $this->get_settings_group(
            'copycode',
            '.mdp-couponer-code a.mdp-couponer-text',
            [ 'couponer_general_coupon_style' => 'copy_code' ],
            [
                'margin'        => true,
                'padding'       => true,
                'color'         => true,
                'typography'    => true,
                'text-shadow'   => true,
                'alignment'     => true,
                'background'    => true,
                'border'        => true,
                'border-radius' => true,
                'box-shadow'    => true,
            ]
        );

    }

    /**
     * Register Controls for Expand Code Message
     *
     * @since 1.0.0
     * @access private
     */
    private function controls_expandcode() {

        $this->add_control(
            'header_expand_text',
            [
                'label'     => esc_html__( 'Expand Message', 'couponer-elementor' ),
                'type'      => Controls_Manager::HEADING,
                'separator' => 'before',
                'condition' => [ 'couponer_general_coupon_style' => 'expand' ]
            ]
        );

        /** Expand code text. */
        $this->get_settings_group(
            'expandcode',
            '.mdp-couponer-expand-text',
            [ 'couponer_general_coupon_style' => 'expand' ],
            [
                'width'         => true,
                'margin'        => true,
                'padding'       => true,
                'color'         => true,
                'typography'    => true,
                'text-shadow'   => true,
                'alignment'     => true,
                'background'    => true,
                'border'        => true,
                'border-radius' => true,
                'box-shadow'    => true,
            ]
        );

    }

    /**
     * Register Controls for Coupon Icon Styles
     *
     * @since 1.0.0
     * @access private
     */
    private function controls_codeicon() {

        $this->add_control(
            'header_code_icon',
            [
                'label'     => esc_html__( 'Code Icon', 'couponer-elementor' ),
                'type'      => Controls_Manager::HEADING,
                'separator' => 'before',
                'condition' => [ 'couponer_general_icon_type' => [ 'icon', 'image' ] ]
            ]
        );

        /** Icon Style */
        $this->get_settings_group(
            'iconcode',
            '.mdp-couponer-code-icon',
            [ 'couponer_general_icon_type' => [ 'icon', 'image' ] ],
            [
                'margin'        => true,
                'padding'       => true,
                'color'         => true,
                'typography'    => true,
                'text-shadow'   => true,
                'alignment'     => false,
                'background'    => true,
                'border'        => true,
                'border-radius' => true,
                'box-shadow'    => true,
            ]
        );

    }

    /**
     * Register Controls for Coupon Button
     *
     * @since 1.0.0
     * @access private
     */
    private function controls_tabs_button() {

        $this->start_controls_tabs( 'tabs-couponer-button' );

        $this->controls_tab_button_normal();

        $this->controls_tab_button_hover();

        $this->end_controls_tabs();

    }

    /**
     * Register Controls for Coupon Button :hormal
     *
     * @since 1.0.0
     * @access private
     */
    private function controls_tab_button_normal() {

        $this->start_controls_tab( 'tab_button_normal', [ 'label' => esc_html__( 'Normal', 'couponer-elementor' ) ] );

        $this->get_settings_group(
            'button',
            '.mdp-couponer-link',
            [],
            [ 'margin' => true, 'padding' => true ]
        );

        $this->get_settings_group(
            'button',
            '.mdp-couponer-link > a',
            [],
            [ 'color' => true, 'typography' => true, 'text-shadow' => true ]
        );

        $this->get_settings_group(
            'button',
            '.mdp-couponer-link-align',
            [],
            [ 'alignment' => true ]
        );

        $this->get_settings_group(
            'button',
            '.mdp-couponer-link-align',
            [],
            [ 'background' => true, 'border' => true, 'border-radius' => true, 'box-shadow' => true ]
        );

        /** Header Button Icon. */
        $this->add_control(
            'header_button_icon',
            [
                'label'     => esc_html__( 'Button Icon', 'couponer-elementor' ),
                'type'      => Controls_Manager::HEADING,
                'separator' => 'before',
            ]
        );

        $this->get_settings_group(
            'buttonicon',
            '.mdp-couponer-button-icon',
            [],
            [
                'margin'        => true,
                'padding'       => true,
                'color'         => true,
                'typography'    => true,
                'text-shadow'   => true,
                'background'    => true,
                'border'        => true,
                'border-radius' => true,
                'box-shadow'    => true,
            ]
        );

        $this->end_controls_tab();

    }

    /**
     * Register Controls for Coupon Button :hover
     *
     * @since 1.0.0
     * @access private
     */
    private function controls_tab_button_hover() {

        $this->start_controls_tab( 'tab_button_hover', [ 'label' => esc_html__( 'Hover', 'couponer-elementor' ) ] );

        $this->get_settings_group(
            'buttonhover',
            '.mdp-couponer-link:hover',
            [],
            [
                'margin'        => true,
                'padding'       => true,
                'color'         => true,
                'typography'    => true,
                'text-shadow'   => true,
                'alignment'     => true,
                'background'    => true,
                'border'        => true,
                'border-radius' => true,
                'box-shadow'    => true,
            ]
        );

        /** Header Button Icon. */
        $this->add_control(
            'header_button_icon_hover',
            [
                'label'     => esc_html__( 'Button Icon', 'couponer-elementor' ),
                'type'      => Controls_Manager::HEADING,
                'separator' => 'before',
            ]
        );

        $this->get_settings_group(
            'buttoniconhover',
            '.mdp-couponer-link:hover .mdp-couponer-button-icon',
            [],
            [
                'margin'        => true,
                'padding'       => true,
                'color'         => true,
                'typography'    => true,
                'text-shadow'   => true,
                'alignment'     => false,
                'background'    => true,
                'border'        => true,
                'border-radius' => true,
                'box-shadow'    => true,
            ]
        );

        $this->end_controls_tab();
    }

}
